/*
 * Licensed from AIAIOO Labs.
 * Copyright holder :  Aiaioo Labs, cohan@aiaioo.com, (2008, 2009, 2010).
 * 
*
*  The software source code in this file and the other files in this library
*  are covered by a license from Aiaioo Labs.
* 
 * The terms and conditions of the license
 * may be obtained from Aiaioo Labs, Bangalore, India (cohan@aiaioo.com).
*/

package com.aiaioo.document;

import java.util.List;

import com.aiaioo.sentence.Paragraph;

public interface Document extends Paragraph {

	/**
	 * Returns the paragraphs in this Document object.
	 * @return the paragraphs encapsulated in this Document object
	 */
	public List<Paragraph> getParagraphs();

	/**
	 * Returns the plain text encapsulated in this Document object
	 * line by line.
	 * 
	 * When the text has all been returned, the method returns a null.
	 * 
	 * This is the preferred method to read large documents where you
	 * don't want the whole document to reside in memory at one time.
	 * 
	 * @return the plain text encapsulated in this Document object
	 */
	public String readLine();

	/**
	 * Resets the line reading so the document can be read again.
	 */
	public void resetReading();
}
