/*
 * Licensed from AIAIOO Labs.
 * Copyright holder :  Aiaioo Labs, cohan@aiaioo.com, (2008, 2009, 2010).
 * 
*
*  The software source code in this file and the other files in this library
*  are covered by a license from Aiaioo Labs.
* 
 * The terms and conditions of the license
 * may be obtained from Aiaioo Labs, Bangalore, India (cohan@aiaioo.com).
*/

package com.aiaioo.document;

import java.util.ArrayList;
import java.util.List;

import com.aiaioo.sentence.Paragraph;
import com.aiaioo.sentence.Sentence;
import com.aiaioo.sentence.SimpleParagraph;
import com.aiaioo.sentence.TextSegment;
import com.aiaioo.util.Pinned;
import com.aiaioo.util.Token;

/**
 * Class implementing a container for text to be processed.
 * A document should be created only using the 'getInstance' static method.
 * 
 * @author Cohan Sujay Carlos (cohan@aiaioo.com)
 * @version $Revision: 1.0 $ */

public class StringDocument implements Document, Pinned {

	protected StringBuffer buff;
	private boolean isRead = false;
	private long position;

	/**
	 * Constructor that leaves the buff empty
	 */
	protected StringDocument() {
	}

	/**
	 * Constructor
	 * @param buffer the text to be encapsulated
	 */
	protected StringDocument(StringBuffer buffer) {
		if (buffer == null)
			this.buff = new StringBuffer();
		else
			this.buff = buffer;
	}

	/**
	 * Constructor
	 * @param buffer the text to be encapsulated
	 */
	protected StringDocument(String buffer) {
		if (buffer == null)
			this.buff = new StringBuffer();
		else
			this.buff = new StringBuffer(buffer);
	}

	/**
	 * Returns the plain text encapsulated in this Document object
	 * @return the plain text encapsulated in this Document object
	 */
	public String readLine() {
		if (buff == null || isRead)
			return null;
		String str = buff.toString();
		isRead = true;
		return str;
	}
	
	/**
	 * Returns the plain text encapsulated in this Document object
	 * @return the plain text encapsulated in this Document object
	 */
	public StringBuffer getCharacters() {
		return buff;
	}

	public List<Token> getWords() {
		return new ArrayList<Token>();
	}

	public List<String> getRawSentences() {
		String[] segments = this.buff.toString().split("\n");
		ArrayList<String> list = new ArrayList<String>();
		for (String segment : segments)
			list.add(segment);
		return list;
	}
	
	public List<Sentence> getSentences() {
		ArrayList<Sentence> sentences = new ArrayList<Sentence>();
		return sentences;
	}

	public List<Paragraph> getParagraphs() {
		ArrayList<Paragraph> paragraphs = new ArrayList<Paragraph>();
		paragraphs.add(new SimpleParagraph(getSentences()));
		return paragraphs;
	}
	
	public boolean matches(TextSegment other) {
		for (Sentence sentence : getSentences()) {
			if (sentence.matches(other))
				return true;
		}
		return false;
	}
	
	public String toString() {
		return buff.toString();
	}

	public int size() {
		int sum = 0;
		for (Sentence sentence : this.getSentences()) {
			sum += sentence.size();
		}
		return sum;
	}

	@Override
	public long getLength() {
		return buff.length();
	}

	@Override
	public long getPosition() {
		return position;
	}

	@Override
	public void setPosition(long position) {
		this.position = position;
	}

	@Override
	public void setLength(long position) {
		for ( ; position > buff.length() ; --position)
			buff.append(' ');
	}
	
	/**
	 * Resets the line reading so the document can be read again.
	 */
	@Override
	public void resetReading() {
		isRead = false;
	}

}
