/*
 * Licensed from AIAIOO Labs.
 * Copyright holder :  Aiaioo Labs, cohan@aiaioo.com, (2008, 2009, 2010).
 * 
*
*  The software source code in this file and the other files in this library
*  are covered by a license from Aiaioo Labs.
* 
 * The terms and conditions of the license
 * may be obtained from Aiaioo Labs, Bangalore, India (cohan@aiaioo.com).
*/

package com.aiaioo.segmenter;

import java.util.ArrayList;
import java.util.List;

import com.aiaioo.document.Document;
import com.aiaioo.util.PinnedToken;
import com.aiaioo.util.Token;

public class SpaceTokenizer implements Tokenizer {

	protected static final String SPECIAL_CHARACTERS[] = {" ", "\t", "\n"};

	protected StringBuffer buffer;
	protected Document document;
	int position = 0;
	
	public SpaceTokenizer(Document document) {
		this.document = document;
	}
	
	public SpaceTokenizer(StringBuffer document) {
		this.buffer = document;
	}
	
	public SpaceTokenizer(String document) {
		this.buffer = new StringBuffer(document);
	}
	
	public Token getNextLowercase() {
		return getNext(true);
	}

	public Token getNext() {
		return getNext(false);
	}
	
	protected Token getNext(boolean isLowercase) {
		bringIntoBuffer();
		if (buffer != null && buffer.length() > 0) {
			eatWhitespace();

			// Find first token
			int index = buffer.length();
			String breakingString = null;
			for (int i = 0 ; i < SPECIAL_CHARACTERS.length ; ++i) {
				int temp_index = buffer.indexOf(SPECIAL_CHARACTERS[i]);
				if (temp_index != -1 && temp_index < index) {
					index = temp_index;
					breakingString = SPECIAL_CHARACTERS[i];
				}
			}

			// Delete first token from the buffer
			String temp = null;
			//int step = 0;
			if (index == buffer.length()) {
				temp = buffer.toString();
				advancePosition(buffer.length());
				buffer.delete(0, buffer.length());
			} else if (index == 0) {
				temp = breakingString;
				advancePosition(breakingString.length());
				buffer.delete(0, breakingString.length());
			} else {
				temp = buffer.substring(0, index);
				if (breakingString == SPECIAL_CHARACTERS[0] || breakingString == SPECIAL_CHARACTERS[1] || breakingString == SPECIAL_CHARACTERS[2]) {
					advancePosition(index+1);
					buffer.delete(0, index+1);
				} else {
					advancePosition(index);
					buffer.delete(0, index);
				}
			}
			return new PinnedToken(isLowercase ? temp.toLowerCase() : temp, position-temp.length());
		} else {
			return null;
		}
	}

	private void advancePosition(int value) {
		position += value;
	}

	private void eatWhitespace() {
		bringIntoBuffer();
		while (buffer != null && (buffer.indexOf(SPECIAL_CHARACTERS[0]) == 0 || buffer.indexOf(SPECIAL_CHARACTERS[1]) == 0 || buffer.indexOf(SPECIAL_CHARACTERS[2]) == 0)) {
			advancePosition(1);
			buffer.delete(0, 1);
		}
	}

	public boolean hasNext() {
		eatWhitespace();
		bringIntoBuffer();
		return buffer != null && buffer.length() > 0;
	}

	private void bringIntoBuffer() {
		while ((buffer == null || buffer.length() == 0) && document != null) {
			if (position > 0)
				advancePosition(1); // add one for the newline
			String line = document.readLine();
			if (line != null) {
				buffer = new StringBuffer(line);
			} else
				break;
		}
	}

	public List<Token> getTokens() {
		ArrayList<Token> tokens = new ArrayList<Token>();
		while (hasNext())
			tokens.add(getNext());
		return tokens;
	}

	public List<Token> getTokensLowercase() {
		ArrayList<Token> tokens = new ArrayList<Token>();
		while (hasNext())
			tokens.add(getNextLowercase());
		return tokens;
	}

	public static void main(String[] args) {
    	StringBuffer buffer = new StringBuffer("Hi, this isn't John who'll be you're guide's!");
		SpaceTokenizer t = new SpaceTokenizer(buffer);
    	while(t.hasNext()) {
    		Token token = t.getNext();
    		System.err.println(token);
    	}

	}
}
