package com.aiaioo.sentence;

import java.util.ArrayList;
import java.util.List;

import com.aiaioo.util.Pinned;
import com.aiaioo.util.PinnedToken;
import com.aiaioo.util.Range;
import com.aiaioo.sentence.TextSegment;
import com.aiaioo.util.Token;

public class BufferedTextSegment extends SimpleTextSegment {
	protected StringBuffer stringBuffer = new StringBuffer();
	protected ArrayList<Integer> charToWordMapping = new ArrayList<Integer>();

	public BufferedTextSegment(TextSegment segment) {
		super(segment.getWords());
		mapCharsToWords(segment);
	}

	private void mapCharsToWords(TextSegment segment) {
		if (segment instanceof Pinned) {
			this.stringBuffer = segment.getCharacters();
			int count = 0;
			PinnedToken lastToken = null;
			for (Token token : segment.getWords()) {
				PinnedToken pt = ((PinnedToken)token);
				long position = pt.getPosition();
				long lastPosition = (lastToken == null ? 0 : lastToken.getPosition()+lastToken.getLength());
				for (long i = lastPosition ; i < position ; ++i) {
					charToWordMapping.add(-1);
				}
				for (long i = position ; i < position+pt.getLength() ; ++i) {
					charToWordMapping.add(count);
				}
				lastToken = pt;
			}
		} else {
			int count = 0;
			int position = 0;
			for (Token token : segment.getWords()) {
				position = mapPositionsInWord(count, position, token);
				++count;
			}
		}
		//if (charToWordMapping.size() > 0)
		//	System.err.println(charToWordMapping.size() + " " + stringBuffer.length());
	}
	
	private int mapPositionsInWord(int count, int position, Token token) {
		if (count > 0) {
			stringBuffer.append(' ');
			charToWordMapping.add(-1);
			++position; // add one whitespace
		}
		int wordLength = token.getValue().length();
		for (int i = position ; i < position+wordLength ; ++i) {
			charToWordMapping.add(count);
		}
		this.stringBuffer.append(token);
		return position;
	}
	
	public int length() {
		return stringBuffer.length();
	}
	
	public List<Token> getWords() {
		return words;
	}
	
	public StringBuffer getCharacters() {
		if (this.stringBuffer != null)
			return this.stringBuffer;
		return new StringBuffer();
	}

	/**
	 * Returns the plain text that constitutes this text segment.
	 * 
	 * This method can be used to get a small text segment
	 * in its entirety for further processing.
	 * 
	 * It is important to bear in mind that the entire
	 * text segment will reside in memory.
	 * 
	 * @return the plain text encapsulated in this text segment
	 */
	public StringBuffer getCharacters(Range range) {
		if (this.stringBuffer != null)
			return new StringBuffer(stringBuffer.substring((int)range.getStart(), (int)range.getEnd()));
		return new StringBuffer();
	}
	
	public void setCharacters(int startPosition, int endPosition, String value) {
		if (startPosition > this.stringBuffer.length()) {
			stringBuffer.ensureCapacity(startPosition);
		}
		if (endPosition > this.stringBuffer.length()) {
			stringBuffer.ensureCapacity(endPosition);
		}
		while (stringBuffer.length() < endPosition) {
			stringBuffer.append(' ');
		}
		stringBuffer.replace(startPosition, endPosition+1, value);
	}

	public void setWord(int position, String value) {
		if (position >= words.size()) {
			stringBuffer.append(' ');
			for (int i = words.size() ; i < position ; ++i) {
				words.add(new PinnedToken("", stringBuffer.length()));
				stringBuffer.append(' ');
				charToWordMapping.add(-1);
			}
			words.add(new PinnedToken(value, stringBuffer.length()));
			stringBuffer.append(value);
			charToWordMapping.add(words.size()-1);
		} else {
			Token pt = words.get(position);
			if (pt instanceof PinnedToken) {
				words.set(position, new PinnedToken(value, ((PinnedToken)pt).getPosition()));
			} else {
				words.set(position, new Token(value));
			}
		}
	}

	/*
	private void adjustFollowingPositions(int startPosition, int size, int change) {
		if (startPosition >= charToWordMapping.size()) {
			for (long i = charToWordMapping.size() ; i <= startPosition ; ++i) {
				charToWordMapping.add(-1);
			}
			for (long i = 0 ; i < size ; ++i) {
				charToWordMapping.add(textSegment.getWords().size());
			}
			return;
		}
		int wordPosition = charToWordMapping.get(startPosition);
		if (wordPosition > 0)
			for (int i = wordPosition ; i < textSegment.getWords().size() ; ++i) {
				long position = ((PinnedToken)textSegment.getWords().get(i)).getPosition();
				((PinnedToken)textSegment.getWords().get(i)).setPosition(position+change);
			}
	}
	*/

	public void deleteCharacters(int startPosition, int endPosition, boolean removeTrailingSpaces) {
		stringBuffer.delete(startPosition, endPosition+1);
		if (removeTrailingSpaces) {
			while (isWhitespace(stringBuffer.charAt(stringBuffer.length()-1)))
				stringBuffer.delete(stringBuffer.length()-1, stringBuffer.length());
		}
	}
	private boolean isWhitespace(char ch) {
		return ch == ' ' || ch == '\n' || ch == '\r' || ch == '\t';
	}

	public boolean equals(Object other) {
		if (other == null || !(other instanceof TextSegment))
			return false;
		return this.toString().equals(other.toString());
	}

	public int hashCode() {
		return this.toString().hashCode();
	}

	public String toString() {
		return this.stringBuffer.toString();
	}
}
