/*
 * Licensed from AIAIOO Labs.
 * Copyright holder :  Aiaioo Labs, cohan@aiaioo.com, (2008, 2009, 2010).
 * 
*
*  The software source code in this file and the other files in this library
*  are covered by a license from Aiaioo Labs.
* 
 * The terms and conditions of the license
 * may be obtained from Aiaioo Labs, Bangalore, India (cohan@aiaioo.com).
*/

package com.aiaioo.sentence;

import java.util.ArrayList;
import java.util.List;

import com.aiaioo.util.Token;

public class SimpleTextSegment implements TextSegment {
	protected List<Token> words = new ArrayList<Token>();

	public SimpleTextSegment(String text) {
	}
	
	public SimpleTextSegment(List<? extends Token> words) {
		this.words.addAll(words);
	}

	public SimpleTextSegment(Token word) {
		this.words.add(word);
	}

	public SimpleTextSegment(Token[] words) {
		for (int i = 0 ; i < words.length ; ++i)
			this.words.add(words[i]);
	}

	public List<? extends Token> getWords() {
		return words;
	}

	public void setWords(List<Token> words) {
		this.words = words;
	}

	public boolean equals(Object other) {
		if (other == null || !(other instanceof SimpleTextSegment))
			return false;
		return this.toString().equals(other.toString());
	}

	public int hashCode() {
		return this.toString().hashCode();
	}

	public String toString() {
		StringBuffer buffer = getCharacters();
		return buffer.toString();
	}

	public StringBuffer getCharacters() {
		StringBuffer buffer = new StringBuffer();
		for (Token word : words) {
			buffer.append(word.toString());
			buffer.append(' ');
		}
		if (buffer.length() > 0 && buffer.charAt(buffer.length()-1) == ' ') {
			buffer.deleteCharAt(buffer.length()-1);
		}
		return buffer;
	}

	public boolean matches(TextSegment simpleTextSegment) {
		List<? extends Token> segment = simpleTextSegment.getWords();
		Token first = segment.get(0);
		for (int i = 0 ; i <= words.size() - segment.size() ; ++i) {
			Token token = words.get(i);
			if (token.equals(first)) {
				boolean totalMatch = true;
				for (int j = 1 ; j < segment.size() ; ++j) {
					Token segmentToken = segment.get(j);
					if (!segmentToken.equals(words.get(i + j))) {
						totalMatch = false;
						break;
					}
				}
				if (totalMatch)
					return true;
			}
		}
		return false;
	}
	
	public boolean matchesAtPosition(int index, SimpleTextSegment simpleTextSegment) {
		if (this.words.size() < index + simpleTextSegment.getWords().size())
			return false;

		ArrayList<Token> otherSegment = new ArrayList<Token>();
		otherSegment.addAll(simpleTextSegment.getWords());

		ArrayList<Token> thisSegment = new ArrayList<Token>();
		thisSegment.addAll(words.subList(index, index+otherSegment.size()));
		
		for (int i = 0 ; i < otherSegment.size() ; ++i) {
			if (!thisSegment.get(i).equals(otherSegment.get(i)))
				return false;
		}
		
		return true;
	}
	
	/*
	 * TODO: OPTIMIZE
	 */
	public boolean matchesIgnoreCaseAtPosition(int index, SimpleTextSegment simpleTextSegment) {
		if (this.words.size() < index + simpleTextSegment.getWords().size())
			return false;

		ArrayList<Token> otherSegment = new ArrayList<Token>();
		otherSegment.addAll(simpleTextSegment.getWords());

		ArrayList<Token> thisSegment = new ArrayList<Token>();
		thisSegment.addAll(words.subList(index, index+otherSegment.size()));
		
		for (int i = 0 ; i < otherSegment.size() ; ++i) {
			if (!thisSegment.get(i).toString().toLowerCase().equals(otherSegment.get(i).toString().toLowerCase()))
				return false;
		}
		
		return true;
	}
	
	public int size() {
		return words.size();
	}
	
//	public static void main(String[] args) {
//		System.err.println(new Sentence(new ArrayList<Token>()) instanceof TextSegment );
//	}
}
