/*
 * Licensed from AIAIOO Labs.
 * Copyright holder :  Aiaioo Labs, cohan@aiaioo.com, (2008, 2009, 2010).
 * 
*
*  The software source code in this file and the other files in this library
*  are covered by a license from Aiaioo Labs.
* 
 * The terms and conditions of the license
 * may be obtained from Aiaioo Labs, Bangalore, India (cohan@aiaioo.com).
*/

package com.aiaioo.sentence;

import java.util.List;

import com.aiaioo.util.Token;

public interface TextSegment {
	/**
	 * Returns the plain text that constitutes this text segment.
	 * 
	 * This method can be used to get a small text segment
	 * in its entirety for further processing.
	 * 
	 * It is important to bear in mind that the entire
	 * text segment will reside in memory.
	 * 
	 * @return the plain text encapsulated in this text segment
	 */
	public StringBuffer getCharacters();

	/**
	 * Returns the words that constitute this text segment.
	 * 
	 * It is important to bear in mind that the entire
	 * text segment will reside in memory.
	 * 
	 * The particulars of whitespaces will not be available in this method.
	 * 
	 * @return the tokens that constitute this text segment
	 */
	public List<? extends Token> getWords();
	
	/**
	 * Returns true if the other text segment is part of this text segment.
	 * 
	 * @return true if the other text segment is part of this text segment
	 */
	public boolean matches(TextSegment other);
	
	/**
	 * Returns the number of tokens that constitute this text segment.
	 * 
	 * It is important to bear in mind that the entire
	 * text segment will reside in memory.
	 * 
	 * The particulars of whitespaces will not be available in this method.
	 * 
	 * @return the number of tokens that constitute this text segment
	 */
	public int size();
}
