/*
 * Licensed from AIAIOO Labs.
 * Copyright holder :  Aiaioo Labs, cohan@aiaioo.com, (2008, 2009, 2010).
 * 
*
*  The software source code in this file and the other files in this library
*  are covered by a license from Aiaioo Labs.
* 
 * The terms and conditions of the license
 * may be obtained from Aiaioo Labs, Bangalore, India (cohan@aiaioo.com).
*/

package com.aiaioo.util;

import java.util.Collection;
import java.util.List;

import com.aiaioo.segmenter.SpaceTokenizer;

public class StringEntity implements Entity {
	private static final Entity BLANK_ENTITY = new StringEntity("");
	protected NGram<Token> tokens;

	public StringEntity() {
		tokens = new NGram<Token>();
	}
	
	public StringEntity(Token token) {
		this.tokens = new NGram<Token>();
		this.tokens.addToken(token);
	}
	
	public StringEntity(List<Token> tokens) {
		this.tokens = new NGram<Token>(tokens);
	}
	
	public StringEntity(String string) {
		this.tokens = new NGram<Token>(new SpaceTokenizer(string).getTokens());
	}
	
	public int size() {
		return tokens.size();
	}
	
	public boolean hasEntity(StringEntity other) {
		if (other.size() > size())
			return false;
		for (int i = 0 ; i <= size() - other.size() ; ++i) {
			if (tokens.subSequence(i, i+other.size()).equals(other.tokens))
				return true;
		}
		return false;
	}

	/*
	 * TODO: OPTIMIZE
	 */
	public boolean hasEntityIgnoreCase(StringEntity other) {
		if (other.size() > size())
			return false;
		String compareString = other.tokens.toString().toLowerCase();
		for (int i = 0 ; i <= size() - other.size() ; ++i) {
			if (tokens.subSequence(i, i+other.size()).toString().toLowerCase().equals(compareString))
				return true;
		}
		return false;
	}

	/*
	 * TODO: OPTIMIZE
	 */
	public boolean hasEntityIgnoreCase(Collection<StringEntity> others) {
		for (StringEntity other : others) {
			if (other.size() > size())
				continue;
			String compareString = other.tokens.toString().toLowerCase();
			for (int i = 0 ; i <= size() - other.size() ; ++i) {
				if (tokens.subSequence(i, i+other.size()).toString().toLowerCase().equals(compareString))
					return true;
			}
		}
		return false;
	}

	public List<Token> getTokens() {
		return tokens.getTokens();
	}

	public boolean equals(Object f) {
		if (f == null || !( f instanceof StringEntity ))
			return false;
		return ((StringEntity)f).getTokens() == getTokens() || getTokens() != null && matchingTokens(getTokens(),((StringEntity)f).getTokens());
	}
	
	private boolean matchingTokens(List<Token> tokens1, List<Token> tokens2) {
		if (tokens1.size() != tokens2.size())
			return false;
		for (int i = 0 ; i < tokens1.size() ; ++i) {
			if (!tokens1.get(i).equals(tokens2.get(i)))
				return false;
		}
		return true;
	}

	public int hashCode() {
		return this.tokens.hashCode();
	}
	
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		for (Token token : tokens.getTokens()) {
			buffer.append(token);
			buffer.append(' ');
		}
		return buffer.toString().trim();
	}

	public static Entity getBlankString() {
		return BLANK_ENTITY;
	}
}
