package com.aiaioo.api;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.ProtocolException;
import java.net.URL;
import java.util.ArrayList;
import java.util.regex.Pattern;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.aiaioo.util.Logger;

public class Client {

	protected Logger logger = new Logger();
	protected String key;
	protected String hostname;

	public Client() {
		// Replace the following with your own server
		this.hostname = "www.aiaioo.com";
		// Replace the following with your own API key
		this.key = "5BpebPmMqGBp_guest@aiaioo.com_6kY0ikl5gDOZ";
	}
	
	public String getServerHostname() {
		return this.hostname;
	}
	
	public String getAPIKey() {
		return this.key;
	}
	
	public void setServerHostname(String hostname) {
		this.hostname = hostname;
	}
	
	public void setKey(String key) {
		this.key = key;
	}
	
	public String getKey() {
		return this.key;
	}
	
	public int getVersion() {
		return 1;
	}
	
	public String getVersionDetails() {
		return "1.0.15";
	}
	
	public void checkVersion(String version) throws ClientException {
		String serverVersion = null;
		if (version == null) {
			serverVersion = "0";
		} else {
			serverVersion = version;
		}
		if (Integer.parseInt(serverVersion) > getVersion()) {
			throw new ClientException("The server is running version "+serverVersion+" whereas your Java API is meant for server version "+getVersion()+".  Please upgrade your Java API.");
		}
	}
	
	public void checkLatestClientVersion(String version) throws ClientException {
		String serverVersion = null;
		if (version == null) {
			serverVersion = getVersionDetails();
		} else {
			serverVersion = version;
		}
		if (!serverVersion.equals(getVersionDetails())) {
			logger.println("The latest Java API version is "+serverVersion+" whereas your Java API is still running version "+getVersionDetails()+".  Please upgrade your Java API.");
		}
	}
	
	protected Document parseXmlFile(URL url, String parameters) {
		//get the factory
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	
		try {
	
			//Using factory get an instance of document builder
			DocumentBuilder db = dbf.newDocumentBuilder();
	
			HttpURLConnection urlc = null;
			try
			{
				urlc = (HttpURLConnection) url.openConnection();
				try
				{
					urlc.setRequestMethod("POST");
				} catch (ProtocolException e)
				{
					logger.println("Shouldn't happen: HttpURLConnection doesn't support POST??" + e);
					logger.log(e);
				}
				urlc.setDoOutput(true);
				urlc.setDoInput(true);
				urlc.setUseCaches(false);
				urlc.setAllowUserInteraction(false);
				//urlc.setRequestProperty("Content-type", "text/xml; charset=" + "UTF-8");
				OutputStream out = urlc.getOutputStream();
				try
				{
					BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(out, "UTF-8"));
					writer.write(parameters);
					writer.flush();
					writer.close();
				} catch (IOException e) {
					logger.println("IOException while posting data" + e);
					logger.log(e);
				} finally {
					if (out != null)
						out.close();
				}
				java.io.BufferedInputStream in = null;
				try
				{
					in = new java.io.BufferedInputStream(urlc.getInputStream());
	
					//BufferedReader reader = new BufferedReader(new InputStreamReader(in));
					//String line = null;
					//while ((line = reader.readLine()) != null) {
					//	logger.println(line);
					//}
					
					InputSource is = new InputSource(in);
					is.setEncoding("UTF-8");
					 
					//parse using builder to get DOM representation of the XML file
					Document dom = db.parse(is);
					
					return dom;
	
				} catch (IOException e) {
					logger.println("IOException while reading response " + e);
					logger.log(e);
				} finally {
					if (in != null)
						in.close();
				}
			} catch (IOException e) {
				logger.println("Connection error (can't connect to the server at " + url + "): " + e);
				logger.log(e);
			} finally {
				if (urlc != null)
					urlc.disconnect();
			}
		}catch(ParserConfigurationException pce) {
			logger.log(pce);
		}catch(SAXException se) {
			logger.log(se);
		}
		return null;
	}

	protected String escapeXml(String item) {
		return "<![CDATA[" + stripNonValidXMLCharacters(    item.replaceAll("%", "%2525").replaceAll("&", "%2526").replaceAll(Pattern.quote("+"),"%252B").replaceAll("]]>", "]]>]]%2526gt;<![CDATA[")   ) + "]]>";
	}

	protected String escapeURL(String item) {
		return item.replaceAll("%", "%2525").replaceAll("&", "%2526").replaceAll(Pattern.quote("+"),"%252B");
	}
	
	 /**
     * This method ensures that the output String has only
     * valid XML unicode characters as specified by the
     * XML 1.0 standard. For reference, please see
     * <a href="http://www.w3.org/TR/2000/REC-xml-20001006#NT-Char">the
     * standard</a>. This method will return an empty
     * String if the input is null or empty.
     *
     * @param in The String whose non-valid characters we want to remove.
     * @return The in String, stripped of non-valid characters.
	  *
	  * This code was taken from Mark McLaren's blog - http://blog.mark-mclaren.info/2007/02/invalid-xml-characters-when-valid-utf8_5873.html
	  * 
     */
    public String stripNonValidXMLCharacters(String in) {
        StringBuffer out = new StringBuffer(); // Used to hold the output.
        char current; // Used to reference the current character.

        if (in == null || ("".equals(in))) return ""; // vacancy test.
        for (int i = 0; i < in.length(); i++) {
            current = in.charAt(i); // NOTE: No IndexOutOfBoundsException caught here; it should not happen.
            if ((current == 0x9) ||
                (current == 0xA) ||
                (current == 0xD) ||
                ((current >= 0x20) && (current <= 0xD7FF)) ||
                ((current >= 0xE000) && (current <= 0xFFFD)) ||
                ((current >= 0x10000) && (current <= 0x10FFFF)))
                out.append(current);
        }
        return out.toString();
    }

	public void performVersioningChecks(Document document) throws ClientException {
		Node versionNode = document.getChildNodes().item(0).getChildNodes().item(2);
		String version = versionNode == null ? null : versionNode.getTextContent();
		checkVersion(version);
		Node latestClientVersionNode = document.getChildNodes().item(0).getChildNodes().item(3);
		String latestClientVersion = latestClientVersionNode == null ? null : latestClientVersionNode.getTextContent();
		checkLatestClientVersion(latestClientVersion);
	}

	public String convertToXML(ArrayList<String> list) {
		String str;
		synchronized (list) {
			if (list.size() == 1) {
				str = escapeURL(list.get(0));
			} else {
				StringBuffer buffer = new StringBuffer("<d>");
				for (String item : list) {
					buffer.append("<s>");
					buffer.append(escapeXml(item));
					buffer.append("</s>");
				}
				buffer.append("</d>");
				str = buffer.toString();
			}
		}
		return str;
	}

}
