package com.aiaioo.api;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.aiaioo.document.StringDocument;
import com.aiaioo.intention.DocumentIntention;
import com.aiaioo.intention.EntityIntention;
import com.aiaioo.intention.Intention;
import com.aiaioo.intention.IntentionType;
import com.aiaioo.sentence.TextSegment;
import com.aiaioo.util.Entity;
import com.aiaioo.util.StringEntity;

public class VakIntentClient extends Client {

	String url = "http://"+getServerHostname()+"/apis/vakintent/";

	public VakIntentClient() {
		super();
	}
	
	public void setServerHostname(String hostname) {
		super.setServerHostname(hostname);
		url = "http://"+getServerHostname()+"/apis/vakintent/";
	}
	
	public ArrayList<Intention> getIntentions(ArrayList<String> list) throws ClientException {
		return getIntentionsForObject(list, null);
	}
	
	public ArrayList<Intention> getIntentionsForObject(ArrayList<String> list, String objectEntity) throws ClientException {
		ArrayList<Intention> intentions = new ArrayList<Intention>();
		if (list == null || list.size() == 0)
			return intentions;
		String str = convertToXML(list);
		String params = "key="+key+"&v="+getVersionDetails()+"&text="+str;
		if (objectEntity != null) {
			params = params + "&object=" + objectEntity;
		}
		try {
			Document document = parseXmlFile(new java.net.URL(url), params);
			if (document != null) {
				performVersioningChecks(document);
				int code = Integer.parseInt(document.getChildNodes().item(0).getChildNodes().item(0).getTextContent());
				if (code == 0) {
					// request processed successfully
					Node documents = document.getChildNodes().item(0).getChildNodes().item(1).getChildNodes().item(0);
					for (int i = 0 ; i < documents.getChildNodes().getLength() ; ++i) {
						Element doc = (Element)documents.getChildNodes().item(i);

						//int index = Integer.parseInt(doc.getChildNodes().item(0).getTextContent());
						String text = doc.getChildNodes().item(1).getTextContent();
						String intention = doc.getChildNodes().item(2).getTextContent();
						String polarity = doc.getChildNodes().item(3).getTextContent();

						DocumentIntention di = new DocumentIntention();
						di.setText(com.aiaioo.document.DocumentBuilder.getInstance(text));
						di.setIntentionType(IntentionType.getIntentionType(intention));
						di.setNegated("negative".equals(polarity));

						intentions.add(di);
						
						NodeList sentences = doc.getElementsByTagName("sentence");
						for (int j = 0 ; j < sentences.getLength() ; ++j) {
							Node sentence = sentences.item(j);
							
							//int indexS = Integer.parseInt(sentence.getChildNodes().item(0).getTextContent());
							String textS = sentence.getChildNodes().item(1).getTextContent();
							String intentionS = sentence.getChildNodes().item(2).getTextContent();
							String polarityS = sentence.getChildNodes().item(3).getTextContent();

							EntityIntention ei = new EntityIntention();
							TextSegment sent = com.aiaioo.document.DocumentBuilder.getInstance(textS);
							ei.setText(sent);
							ei.setIntentionType(IntentionType.getIntentionType(intentionS));
							ei.setNegated("negative".equals(polarityS));

							di.addSentenceIntention(sent, ei);
							
							if (sentence.getChildNodes().getLength() > 4 && "objects".equals(sentence.getChildNodes().item(4).getNodeName())) {
								Node objects = sentence.getChildNodes().item(4);
								for (int k = 0 ; k < objects.getChildNodes().getLength() ; ++k) {
									Node object = objects.getChildNodes().item(k);
									
									String textO = object.getChildNodes().item(0).getTextContent();
									String polarityO = object.getChildNodes().item(1).getTextContent();
	
									ei.addObjectOfIntention(new StringEntity(textO), "negative".equals(polarityO));
	
								}
							}
							if (sentence.getChildNodes().getLength() > 5 && "holders".equals(sentence.getChildNodes().item(5).getNodeName())) {
								Node holders = sentence.getChildNodes().item(5);
								for (int k = 0 ; k < holders.getChildNodes().getLength() ; ++k) {
									Node object = holders.getChildNodes().item(k);
									
									String textO = object.getChildNodes().item(0).getTextContent();
									String polarityO = object.getChildNodes().item(1).getTextContent();
	
									ei.addHolderOfIntention(new StringEntity(textO), "negative".equals(polarityO));
	
								}
							}
							if (sentence.getChildNodes().getLength() > 5 && "alternatives".equals(sentence.getChildNodes().item(5).getNodeName())) {
								Node alternatives = sentence.getChildNodes().item(5);
								for (int k = 0 ; k < alternatives.getChildNodes().getLength() ; ++k) {
									Node object = alternatives.getChildNodes().item(k);
									
									String textO = object.getChildNodes().item(0).getTextContent();
									String polarityO = object.getChildNodes().item(1).getTextContent();
	
									ei.addAlternativeOfIntention(new StringEntity(textO), "negative".equals(polarityO));
	
								}
							}
							if (sentence.getChildNodes().getLength() > 6 && "holders".equals(sentence.getChildNodes().item(6).getNodeName())) {
								Node holders = sentence.getChildNodes().item(6);
								for (int k = 0 ; k < holders.getChildNodes().getLength() ; ++k) {
									Node object = holders.getChildNodes().item(k);
									
									String textO = object.getChildNodes().item(0).getTextContent();
									String polarityO = object.getChildNodes().item(1).getTextContent();
	
									ei.addHolderOfIntention(new StringEntity(textO), "negative".equals(polarityO));
	
								}
							}
							if (sentence.getChildNodes().getLength() > 6 && "alternatives".equals(sentence.getChildNodes().item(6).getNodeName())) {
								Node alternatives = sentence.getChildNodes().item(6);
								for (int k = 0 ; k < alternatives.getChildNodes().getLength() ; ++k) {
									Node object = alternatives.getChildNodes().item(k);
									
									String textO = object.getChildNodes().item(0).getTextContent();
									String polarityO = object.getChildNodes().item(1).getTextContent();
	
									ei.addAlternativeOfIntention(new StringEntity(textO), "negative".equals(polarityO));
	
								}
							}
						}

					}
				} else {
					String error = document.getChildNodes().item(0).getChildNodes().item(1).getTextContent();
					if (code == 4)
						throw new ClientException("API key: "+key+".  "+error);
					throw new ClientException(error);
				}
			} else {
				throw new ClientException("Did not receive data from the server.  This is either an internet connectivity problem or faulty XML was returned.");
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (NumberFormatException ex) {
			ex.printStackTrace();
		}
		return intentions;
	}

	public static void testIntent(VakIntentClient client, ArrayList<String> documents) {
		try {
			
			// The intentions are returned as an array, in the same order.
			ArrayList<Intention> intentions = client.getIntentions(documents);
			if (intentions.get(0).getIntentionType().equals(IntentionType.purchase)) ; else throw new ClientException("incorrect intention returned for 0:0");
			if (intentions.get(1).getIntentionType().equals(IntentionType.sell)) ; else throw new ClientException("incorrect intention returned for 0:1");
			if (intentions.get(2).getIntentionType().equals(IntentionType.compare)) ; else throw new ClientException("incorrect intention returned for 0:2");
			if (intentions.get(3).getIntentionType().equals(IntentionType.accuse)) ; else throw new ClientException("incorrect intention returned for 0:3");
			if (intentions.get(4).getIntentionType().equals(IntentionType.inquire)) ; else throw new ClientException("incorrect intention returned for 0:4");
			
			// This time only intentions that concern the VW object are returned
			ArrayList<Intention> intentionsForObject1 = client.getIntentionsForObject(documents, "VW");
			if (intentionsForObject1.get(0).getIntentionType().equals(IntentionType.purchase)) ; else throw new ClientException("incorrect intention returned for 1:0");
			if (intentionsForObject1.get(1).getIntentionType().equals(IntentionType.other)) ; else throw new ClientException("incorrect intention returned for 1:1");
			if (intentionsForObject1.get(2).getIntentionType().equals(IntentionType.other)) ; else throw new ClientException("incorrect intention returned for 1:2");
			if (intentionsForObject1.get(3).getIntentionType().equals(IntentionType.other)) ; else throw new ClientException("incorrect intention returned for 1:3");
			if (intentionsForObject1.get(4).getIntentionType().equals(IntentionType.other)) ; else throw new ClientException("incorrect intention returned for 1:4");

			// This time only intentions that concern the Gibson object are returned
			ArrayList<Intention> intentionsForObject2 = client.getIntentionsForObject(documents, "gibson");
			if (intentionsForObject2.get(0).getIntentionType().equals(IntentionType.other)) ; else throw new ClientException("incorrect intention returned for 2:0");
			if (intentionsForObject2.get(1).getIntentionType().equals(IntentionType.sell)) ; else throw new ClientException("incorrect intention returned for 2:1");
			if (intentionsForObject2.get(2).getIntentionType().equals(IntentionType.other)) ; else throw new ClientException("incorrect intention returned for 2:2");
			if (intentionsForObject2.get(3).getIntentionType().equals(IntentionType.other)) ; else throw new ClientException("incorrect intention returned for 2:3");
			if (intentionsForObject2.get(4).getIntentionType().equals(IntentionType.other)) ; else throw new ClientException("incorrect intention returned for 2:4");
			
		} catch (ClientException e) {
			System.err.println("Error: "+e.getMessage());
		}
	}

	public static void stressTest(final VakIntentClient client, final ArrayList<String> documents) {
		final int LENGTH = 10;
		Thread t1 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t2 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t3 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t4 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t5 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t6 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t7 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t8 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t9 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t10 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t11 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t12 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t13 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t14 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t15 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t16 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t17 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t18 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t19 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		Thread t20 = new Thread(new Runnable() {
			@Override
			public void run() {
				for (int i = 0 ; i < LENGTH ; ++i) {
					testIntent(client, documents);
				}
			}
		});
		t1.start();
		t2.start();
		t3.start();
		t4.start();
		t5.start();
//		t6.start();
//		t7.start();
//		t8.start();
//		t9.start();
//		t10.start();
//		t11.start();
//		t12.start();
//		t13.start();
//		t14.start();
//		t15.start();
//		t16.start();
//		t17.start();
//		t18.start();
//		t19.start();
//		t20.start();
	}

	public static void main(String[] args) {
		try {
			
			// You can send documents in batches.  You can do that by placing multiple documents
			// in an ArrayList and passing it to the API

			ArrayList<String> documents = new ArrayList<String>();
			documents.add("I would like    to buy a car like that beautiful VW we saw near San Mateo.  I really like it.");
			documents.add("For sale: a new Gibson electric with amplifier.");
			documents.add("The Nokia 600 is much better than the older N72.");
			documents.add("It is not this government's fault that the economy is hardly growing.");
			documents.add("What is the printing resolution of the model 453 printer?");
			documents.add("For rt: a bart with an amplifier.");
			documents.add("我不喜歡新的豐田.");
			documents.add("@dude This is just a test of < special > characters & atleast 4% of 'them' - %$*()/~`\\[]'\"}#- must be here? ]");
			documents.add("Dear Verizon, I have been with your company for over four years");
			documents.add("Dear Verizon, I have been with your ]]> company for over four years");
			
			// First we create an instance of the client

			VakIntentClient client = new VakIntentClient();
			
			// Getting the intention for all the sentences
			
			System.out.println("EXAMPLE1 - NO ENTITIES - Any intentions present are returned");
			ArrayList<Intention> intentions = client.getIntentions(documents);
			System.out.println(intentions);
			
			for (Intention intention : intentions) {
				DocumentIntention ds = (DocumentIntention)intention;
				System.out.println(">> The document text is "+intention.getText());
				Map<TextSegment, Intention> ss = ds.getSentenceIntention();
				for (TextSegment ts : ss.keySet()) {
					HashMap<Entity, Boolean> entityIntention = ((EntityIntention)ss.get(ts)).getObjectsOfIntention();
					System.out.println(">> The sentence text is "+((EntityIntention)ss.get(ts)).getText());
					for (Entity e : entityIntention.keySet()) {
						System.out.println("Entity: " + e + " and intention: " + ((EntityIntention)ss.get(ts)).getIntentionType() + " in sentence "+ts);
					}
					System.out.println(">> The sentence starts at "+((StringDocument)ts).getPosition()+" and has length "+((StringDocument)ts).getLength());
				}
			}

			// This time only intentions that concern the VW object are returned
			
			System.out.println("EXAMPLE2 - 1 ENTITY - The intentions are measured, this time just for VW");
			ArrayList<Intention> intentionsForObject1 = new VakIntentClient().getIntentionsForObject(documents, "VW");
			System.out.println(intentionsForObject1);

			// This time only intentions that concern the Gibson object are returned

			System.out.println("EXAMPLE3 - 1 ENTITY - The intentions are measured, this time just for Gibson");
			ArrayList<Intention> intentionsForObject2 = new VakIntentClient().getIntentionsForObject(documents, "gibson");
			System.out.println(intentionsForObject2);
			
			// This time only intentions that concern the VW or the Gibson object are returned

			System.out.println("EXAMPLE4 - 2 ENTITIES - The intentions are measured, this time for VW and Gibson");
			ArrayList<Intention> intentionsForObject3 = new VakIntentClient().getIntentionsForObject(documents, "vw,gibson");
			System.out.println(intentionsForObject3);

			// Now this is a stress test

			System.out.println("EXAMPLE5 - Round Trip Time Test - This will take around 15 seconds.");
			System.out.println("Commencing performance tests ... making 30 API calls to the server.");
			long startTime = System.currentTimeMillis();

			// make 30 calls 
			for (int i = 0 ; i < 10 ; ++i) { // 10 iterations
				testIntent(client, documents); // of 3 API calls each
			}

			long endTime = System.currentTimeMillis();
			System.out.println("30 API calls with 300 documents completed in "+((endTime - startTime)/1000.0)+" seconds.");
			
			/*
			
			System.out.println("EXAMPLE6 - Stress Test - This will take around 15 seconds.");
			System.out.println("Making those 30 API calls each from 5 threads simultaneously (a total of 150 API calls).  Watch for program termination.");
			
			//for (int i = 0 ; i < 10 ; ++i)
				//stressTest(client, documents);
			 
				 
			*/
			
		} catch (ClientException e) {
			e.printStackTrace();
			System.out.println("Error: "+e.getMessage());
		}
	}

}
