package com.aiaioo.api;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.aiaioo.sentence.TextSegment;
import com.aiaioo.sentiment.DocumentSentiment;
import com.aiaioo.sentiment.EntitySentiment;
import com.aiaioo.sentiment.Sentiment;
import com.aiaioo.sentiment.SentimentType;
import com.aiaioo.util.Entity;
import com.aiaioo.util.StringEntity;

public class VakSentClient extends Client {
	
	String url = "http://"+getServerHostname()+"/apis/vaksent/";

	public VakSentClient() {
		super();
	}
	
	public void setServerHostname(String hostname) {
		super.setServerHostname(hostname);
		url = "http://"+getServerHostname()+"/apis/vaksent/";
	}
	
	public ArrayList<Sentiment> getSentiments(ArrayList<String> list) throws ClientException {
		return getSentimentsForEntity(list, null);
	}
	
	/*
	 * This API still doesn't work for sentiment analysis, hence is private
	 */
	
	private ArrayList<Sentiment> getSentimentsForEntity(ArrayList<String> list, String objectEntity) throws ClientException {
		ArrayList<Sentiment> sentiment = new ArrayList<Sentiment>();
		if (list == null || list.size() == 0)
			return sentiment;
		String str = convertToXML(list);
		String params = "key="+key+"&v="+getVersionDetails()+"&text="+str;
		if (objectEntity != null) {
			params = params + "&object=" + objectEntity;
		}
		try {
			//System.err.println(url+"?"+params);
			Document document = parseXmlFile(new java.net.URL(url), params);
			if (document != null) {
				performVersioningChecks(document);
				int code = Integer.parseInt(document.getChildNodes().item(0).getChildNodes().item(0).getTextContent());
				if (code == 0) {
					// request processed successfully
					Node documents = document.getChildNodes().item(0).getChildNodes().item(1).getChildNodes().item(0);
					for (int i = 0 ; i < documents.getChildNodes().getLength() ; ++i) {
						Element doc = (Element)documents.getChildNodes().item(i);

						//int index = Integer.parseInt(doc.getChildNodes().item(0).getTextContent());
						String text = doc.getChildNodes().item(1).getTextContent();
						String intention = doc.getChildNodes().item(2).getTextContent();

						DocumentSentiment di = new DocumentSentiment();
						di.setText(com.aiaioo.document.DocumentBuilder.getInstance(text));
						di.setSentimentType(SentimentType.getSentimentType(intention));

						sentiment.add(di);
						
						NodeList sentences = doc.getElementsByTagName("sentence");
						for (int j = 0 ; j < sentences.getLength() ; ++j) {
							Node sentence = sentences.item(j);
							
							//int indexS = Integer.parseInt(sentence.getChildNodes().item(0).getTextContent());
							String textS = sentence.getChildNodes().item(1).getTextContent();
							String intentionS = sentence.getChildNodes().item(2).getTextContent();

							EntitySentiment ei = new EntitySentiment();
							TextSegment sent = com.aiaioo.document.DocumentBuilder.getInstance(textS);
							ei.setText(sent);
							ei.setSentimentType(SentimentType.getSentimentType(intentionS));

							di.addSentenceSentiment(sent, ei);
							
							if (sentence.getChildNodes().getLength() > 3 && "entities".equals(sentence.getChildNodes().item(3).getNodeName())) {
								Node objects = sentence.getChildNodes().item(3);
								for (int k = 0 ; k < objects.getChildNodes().getLength() ; ++k) {
									Node object = objects.getChildNodes().item(k);
									
									String textO = object.getChildNodes().item(0).getTextContent();
									String polarityO = object.getChildNodes().item(1).getTextContent();
	
									ei.addEntitySentiment(new StringEntity(textO), new Sentiment(SentimentType.getSentimentType(polarityO)));
	
								}
							}
							if (sentence.getChildNodes().getLength() > 4 && "holders".equals(sentence.getChildNodes().item(4).getNodeName())) {
								Node holders = sentence.getChildNodes().item(4);
								for (int k = 0 ; k < holders.getChildNodes().getLength() ; ++k) {
									Node object = holders.getChildNodes().item(k);
									
									String textO = object.getChildNodes().item(0).getTextContent();
									String polarityO = object.getChildNodes().item(1).getTextContent();
	
									ei.addHolderSentiment(new StringEntity(textO), new Sentiment(SentimentType.getSentimentType(polarityO)));
	
								}
							}
						}

					}
				} else {
					String error = document.getChildNodes().item(0).getChildNodes().item(1).getTextContent();
					if (code == 4)
						throw new ClientException("API key: "+key+".  "+error);
					throw new ClientException(error);
				}
			} else {
				throw new ClientException("Did not receive data from the server.  This is either an internet connectivity problem or faulty XML was returned.");
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (NumberFormatException ex) {
			ex.printStackTrace();
		}
		return sentiment;
	}

	public static void main(String[] args) {
		
		// First we create an instance of the client
		VakSentClient client = new VakSentClient();

		// The following is a demonstration of the Java client
		
		try {
		
			// You can send documents in batches.  You can do that by placing multiple documents
			// in an ArrayList and passing it to the API
			
			ArrayList<String> documents = new ArrayList<String>();
			documents.add("John and not   Bruce said that this is not a bad car.  But they both hated it.");
			documents.add("I deny that it can never be said that this is not a beautiful car.");
			documents.add("The market thinks that Apple and not HTC is going to win.");
			documents.add("The prime minister regrets that his country, though not its parliament, is poor.");
			documents.add("我不喜歡新的豐田.");
			documents.add("No me gusta el nuevo %Toyota%.");
			documents.add("I like testing % & ? $ # + +");

			documents.add("Dont like the service! Seriously sucks!!");
			documents.add("Service so suck ! Good suck , everything suck !");
			documents.add("Your point structure on loyalty program is not good");

			// Getting the sentiment for all the sentences
			
			System.out.println("EXAMPLE1 - NO ENTITIES");
			ArrayList<Sentiment> sentiments = client.getSentiments(documents);
			System.out.println(sentiments);
			
			for (Sentiment sentiment : sentiments) {
				DocumentSentiment ds = (DocumentSentiment)sentiment;
				System.out.println(">> The document is "+sentiment.getText());
				Map<TextSegment, Sentiment> ss = ds.getSentenceSentiment();
				for (TextSegment ts : ss.keySet()) {
					HashMap<Entity, Sentiment> entitySentiment = ((EntitySentiment)ss.get(ts)).getEntitySentiment();
					for (Entity e : entitySentiment.keySet()) {
						System.out.println("Entity: " + e + " and sentiment: " + entitySentiment.get(e).getSentimentType()+" in sentence "+ts);
						if (SentimentType.positive.equals(entitySentiment.get(e).getSentimentType())) {
							System.out.println("This is a positive mention.");
						}
						if (SentimentType.negative.equals(entitySentiment.get(e).getSentimentType())) {
							System.out.println("This is a negative mention.");
						}
						if (SentimentType.neutral.equals(entitySentiment.get(e).getSentimentType())) {
							System.out.println("This is a neutral mention.");
						}
					}
				}
			}

			// Getting the sentiment for the word car

			System.out.println("EXAMPLE2 - 1 ENTITY - car");
			sentiments = client.getSentimentsForEntity(documents, "car");
			System.out.println(sentiments);

			System.out.println("EXAMPLE3 - 3 ENTITIES - car,Apple,Toyota");
			sentiments = client.getSentimentsForEntity(documents, "car,Apple,Toyota");
			System.out.println(sentiments);

			System.out.println("EXAMPLE4 - Case-insensitivity - 3 ENTITIES - car,apple,toyota");
			sentiments = client.getSentimentsForEntity(documents, "car,apple,toyota");
			System.out.println(sentiments);

		} catch (ClientException e) {
			System.err.println("Error: "+e.getMessage());
		}
	}

}
