package com.aiaioo.event;

import java.util.HashMap;

import com.aiaioo.util.Entity;

public class EntityEvent extends Event {
	
	private static final boolean noHolder = false;
	private static final boolean noRecipient = false;
	
	// The maps carry true values if the entity is negated
	protected HashMap<Entity, Boolean> prices = new HashMap<Entity, Boolean>();
	protected HashMap<Entity, Boolean> locations = new HashMap<Entity, Boolean>();
	protected HashMap<Entity, Boolean> objects = new HashMap<Entity, Boolean>();
	protected HashMap<Entity, Boolean> topics = new HashMap<Entity, Boolean>();
	protected HashMap<Entity, Boolean> actors = new HashMap<Entity, Boolean>();
	protected HashMap<Entity, Boolean> experiencers = new HashMap<Entity, Boolean>();

	public EntityEvent() {
	}
		
	public void addObject(Entity entity, boolean negated) {
		objects.put(entity, negated);
	}

	public void addActor(Entity entity, boolean negated) {
		actors.put(entity, negated);
	}

	public void addExperiencer(Entity entity, boolean negated) {
		experiencers.put(entity, negated);
	}

	public void addPrice(Entity entity, boolean negated) {
		prices.put(entity, negated);
	}

	public void addLocation(Entity entity, boolean negated) {
		locations.put(entity, negated);
	}

	
	public HashMap<Entity, Boolean> getObjects() {
		return objects;
	}
	
	public HashMap<Entity, Boolean> getActors() {
		return actors;
	}
	
	public HashMap<Entity, Boolean> getExperiencers() {
		return experiencers;
	}
	
	public String toString() {
		StringBuffer buffer = new StringBuffer(getText().toString());
		buffer.append('\n');
		buffer.append("Event: "+this.getEventType());
		if (this.isNegated()) {
			buffer.append("\t(");
			buffer.append("negated");
			buffer.append(")");
		}
		buffer.append('\n');
		for (Entity entity : actors.keySet()) {
			buffer.append("Actor: "+entity.toString());
			if (actors.get(entity)) {
				buffer.append(" (");
				buffer.append("negated");
				buffer.append(")");
			}
			buffer.append('\n');
		}
		if (!noRecipient) {
			for (Entity entity : experiencers.keySet()) {
				buffer.append("Experiencer: "+entity.toString());
				if (experiencers.get(entity)) {
					buffer.append(" (");
					buffer.append("negated");
					buffer.append(")");
				}
				buffer.append('\n');
			}
		}
		for (Entity entity : objects.keySet()) {
			buffer.append("Object: "+entity.toString());
			if (objects.get(entity)) {
				buffer.append(" (");
				buffer.append("negated");
				buffer.append(")");
			}
			buffer.append('\n');
		}
		if (!noHolder) {
			for (Entity entity : prices.keySet()) {
				buffer.append("Price: "+entity.toString());
				if (actors.get(entity)) {
					buffer.append(" (");
					buffer.append("negated");
					buffer.append(")");
				}
				buffer.append('\n');
			}
		}
		for (Entity entity : locations.keySet()) {
			buffer.append("Location: "+entity.toString());
			if (actors.get(entity)) {
				buffer.append(" (");
				buffer.append("negated");
				buffer.append(")");
			}
			buffer.append('\n');
		}
		return buffer.toString();
	}

	public String toXMLString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("<text>");
		buffer.append(escapeXMLBig(getText().toString()));
		buffer.append("</text>");
		buffer.append("<event>");
		buffer.append((this.getEventType() == null) ? "other" : this.getEventType());
		buffer.append("</event>");
		buffer.append("<polarity>");
		if (this.isNegated()) {
			buffer.append("negative");
		} else {
			buffer.append("positive");
		}
		buffer.append("</polarity>");
		if (actors != null && actors.size() > 0) {
			buffer.append("<actors>");
			for (Entity entity : actors.keySet()) {
				buffer.append("<actor>");
				buffer.append("<value>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</value>");
				buffer.append("<polarity>");
				if (actors.get(entity)) {
					buffer.append("negative");
				} else {
					buffer.append("positive");
				}
				buffer.append("</polarity>");
				buffer.append("</actor>");
			}
			buffer.append("</actors>");
		}
		if (experiencers != null && experiencers.size() > 0) {
			buffer.append("<experiencers>");
			for (Entity entity : experiencers.keySet()) {
				buffer.append("<experiencer>");
				buffer.append("<value>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</value>");
				buffer.append("<polarity>");
				if (experiencers.get(entity)) {
					buffer.append("negative");
				} else {
					buffer.append("positive");
				}
				buffer.append("</polarity>");
				buffer.append("</experiencer>");
			}
			buffer.append("</experiencers>");
		}
		if (objects != null && objects.size() > 0) {
			buffer.append("<objects>");
			for (Entity entity : objects.keySet()) {
				buffer.append("<object>");
				buffer.append("<value>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</value>");
				buffer.append("<polarity>");
				if (objects.get(entity)) {
					buffer.append("negative");
				} else {
					buffer.append("positive");
				}
				buffer.append("</polarity>");
				buffer.append("</object>");
			}
			buffer.append("</objects>");
		}
		if (prices != null && prices.size() > 0) {
			buffer.append("<prices>");
			for (Entity entity : prices.keySet()) {
				buffer.append("<price>");
				buffer.append("<value>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</value>");
				buffer.append("<polarity>");
				if (prices.get(entity)) {
					buffer.append("negative");
				} else {
					buffer.append("positive");
				}
				buffer.append("</polarity>");
				buffer.append("</price>");
			}
			buffer.append("</prices>");
		}
		return buffer.toString();
	}

	public String toWebString() {
		StringBuffer buffer = new StringBuffer(escapeXML(getText().toString()));
		buffer.append(" = [ <font color='blue'> the event is ");
		buffer.append(this.getEventType());
		if (this.isNegated()) {
			buffer.append(" (");
			buffer.append("negated");
			buffer.append(")");
		}
		buffer.append("</font> ]");
		buffer.append("<br/>");
		if (actors != null && actors.size() > 0) {
			buffer.append("<br/>");
			buffer.append("<table>");
			buffer.append("<th>Actors</th><th>Negated?</th>");
			for (Entity entity : actors.keySet()) {
				buffer.append("<tr>").append("<td>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</td>").append("<td>");
				buffer.append(actors.get(entity));
				buffer.append("</td>");
				buffer.append("</tr>");
			}
			buffer.append("</table>");
		}
		if (experiencers != null && experiencers.size() > 0) {
			buffer.append("<br/>");
			buffer.append("<table>");
			buffer.append("<th>Experiencers</th><th>Negated?</th>");
			for (Entity entity : experiencers.keySet()) {
				buffer.append("<tr>").append("<td>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</td>").append("<td>");
				buffer.append(experiencers.get(entity));
				buffer.append("</td>");
				buffer.append("</tr>");
			}
			buffer.append("</table>");
		}
		if (objects != null && objects.size() > 0) {
			buffer.append("<br/>");
			buffer.append("<table>");
			buffer.append("<th>Objects</th><th>Negated?</th>");
			for (Entity entity : objects.keySet()) {
				buffer.append("<tr>").append("<td>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</td>").append("<td>");
				buffer.append(objects.get(entity));
				buffer.append("</td>");
				buffer.append("</tr>");
			}
			buffer.append("</table>");
		}
		return buffer.toString();
	}
}
