package com.aiaioo.intention;

import java.util.HashMap;

import com.aiaioo.util.Entity;

public class EntityIntention extends Intention {
	
	private static final boolean noHolder = false;
	private static final boolean noRecipient = false;
	
	protected HashMap<Entity, Boolean> objectsOfIntention = new HashMap<Entity, Boolean>();
	protected HashMap<Entity, Boolean> alternativesOfIntention = new HashMap<Entity, Boolean>();
	protected HashMap<Entity, Boolean> holdersOfIntention = new HashMap<Entity, Boolean>();
	protected HashMap<Entity, Boolean> recipientsOfIntention = new HashMap<Entity, Boolean>();

	public EntityIntention() {
	}

	public EntityIntention(IntentionType type) {
		super(type);
	}

	public void addObjectOfIntention(Entity entity, boolean negated) {
		objectsOfIntention.put(entity, negated);
	}

	public void addAlternativeOfIntention(Entity entity, boolean negated) {
		alternativesOfIntention.put(entity, negated);
	}

	public void addHolderOfIntention(Entity entity, boolean negated) {
		holdersOfIntention.put(entity, negated);
	}

	public void addRecipientOfIntention(Entity entity, boolean negated) {
		recipientsOfIntention.put(entity, negated);
	}

	public HashMap<Entity, Boolean> getObjectsOfIntention() {
		return objectsOfIntention;
	}
	
	public HashMap<Entity, Boolean> getAlternativesOfIntention() {
		return alternativesOfIntention;
	}
	
	public HashMap<Entity, Boolean> getHoldersOfIntention() {
		return holdersOfIntention;
	}
	
	public HashMap<Entity, Boolean> getRecipientsOfIntention() {
		return recipientsOfIntention;
	}
	
	public String toString() {
		StringBuffer buffer = new StringBuffer(getText().toString());
		buffer.append('\n');
		buffer.append("Intention: "+this.getIntentionType());
		if (this.isNegated()) {
			buffer.append("\t(");
			buffer.append("negated");
			buffer.append(")");
		}
		buffer.append('\n');
		for (Entity entity : objectsOfIntention.keySet()) {
			buffer.append("Object: "+entity.toString());
			if (objectsOfIntention.get(entity)) {
				buffer.append(" (");
				buffer.append("negated");
				buffer.append(")");
			}
			buffer.append('\n');
		}
		for (Entity entity : alternativesOfIntention.keySet()) {
			buffer.append("Alternative: "+entity.toString());
			if (alternativesOfIntention.get(entity)) {
				buffer.append(" (");
				buffer.append("negated");
				buffer.append(")");
			}
			buffer.append('\n');
		}
		if (!noHolder) {
			for (Entity entity : holdersOfIntention.keySet()) {
				buffer.append("Holder: "+entity.toString());
				if (holdersOfIntention.get(entity)) {
					buffer.append(" (");
					buffer.append("negated");
					buffer.append(")");
				}
				buffer.append('\n');
			}
		}
		if (!noRecipient) {
			for (Entity entity : recipientsOfIntention.keySet()) {
				buffer.append("Recipient: "+entity.toString());
				if (recipientsOfIntention.get(entity)) {
					buffer.append(" (");
					buffer.append("negated");
					buffer.append(")");
				}
				buffer.append('\n');
			}
		}
		return buffer.toString();
	}

	public String toXMLString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("<text>");
		buffer.append(escapeXMLBig(getText().toString()));
		buffer.append("</text>");
		buffer.append("<intention>");
		buffer.append((this.getIntentionType() == null) ? "other" : this.getIntentionType());
		buffer.append("</intention>");
		buffer.append("<polarity>");
		if (this.isNegated()) {
			buffer.append("negative");
		} else {
			buffer.append("positive");
		}
		buffer.append("</polarity>");
		if (objectsOfIntention != null && objectsOfIntention.size() > 0) {
			buffer.append("<objects>");
			for (Entity entity : objectsOfIntention.keySet()) {
				buffer.append("<object>");
				buffer.append("<value>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</value>");
				buffer.append("<polarity>");
				if (objectsOfIntention.get(entity)) {
					buffer.append("negative");
				} else {
					buffer.append("positive");
				}
				buffer.append("</polarity>");
				buffer.append("</object>");
			}
			buffer.append("</objects>");
		}
		if (alternativesOfIntention != null && alternativesOfIntention.size() > 0) {
			buffer.append("<alternatives>");
			for (Entity entity : alternativesOfIntention.keySet()) {
				buffer.append("<alternative>");
				buffer.append("<value>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</value>");
				buffer.append("<polarity>");
				if (alternativesOfIntention.get(entity)) {
					buffer.append("negative");
				} else {
					buffer.append("positive");
				}
				buffer.append("</polarity>");
				buffer.append("</alternative>");
			}
			buffer.append("</alternatives>");
		}
		if (!noHolder) {
			if (holdersOfIntention != null && holdersOfIntention.size() > 0) {
				buffer.append("<holders>");
				for (Entity entity : holdersOfIntention.keySet()) {
					buffer.append("<holder>");
					buffer.append("<value>");
					buffer.append(escapeXML(entity.toString()));
					buffer.append("</value>");
					buffer.append("<polarity>");
					if (holdersOfIntention.get(entity)) {
						buffer.append("negative");
					} else {
						buffer.append("positive");
					}
					buffer.append("</polarity>");
					buffer.append("</holder>");
				}
				buffer.append("</holders>");
			}
		}
		if (!noRecipient) {
			if (recipientsOfIntention != null && recipientsOfIntention.size() > 0) {
				buffer.append("<recipients>");
				for (Entity entity : recipientsOfIntention.keySet()) {
					buffer.append("<recipient>");
					buffer.append("<value>");
					buffer.append(escapeXML(entity.toString()));
					buffer.append("</value>");
					buffer.append("<polarity>");
					if (recipientsOfIntention.get(entity)) {
						buffer.append("negative");
					} else {
						buffer.append("positive");
					}
					buffer.append("</polarity>");
					buffer.append("</recipient>");
				}
				buffer.append("</recipients>");
			}
		}
		addPayload(buffer);
		return buffer.toString();
	}

	protected void addPayload(StringBuffer buffer) {
	}

	public String toWebString() {
		StringBuffer buffer = new StringBuffer(escapeXML(getText().toString()));
		buffer.append(" = [ <font color='blue'> the intention is to ");
		buffer.append(this.getIntentionType());
		if (this.isNegated()) {
			buffer.append(" (");
			buffer.append("negated");
			buffer.append(")");
		}
		buffer.append("</font> ]");
		buffer.append("<br/>");
		if (objectsOfIntention != null && objectsOfIntention.size() > 0) {
			buffer.append("<br/>");
			buffer.append("<table>");
			buffer.append("<th>Object of Intention</th><th>Intention</th>");
			for (Entity entity : objectsOfIntention.keySet()) {
				buffer.append("<tr>").append("<td>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</td>").append("<td>");
				buffer.append(this.getIntentionType());
				buffer.append(' ');
				buffer.append(objectsOfIntention.get(entity) ? "(negated)" : "");
				buffer.append("</td>");
				buffer.append("</tr>");
			}
			buffer.append("</table>");
		}
		if (alternativesOfIntention != null && alternativesOfIntention.size() > 0) {
			buffer.append("<br/>");
			buffer.append("<table>");
			buffer.append("<th>Alternative Object of Intention</th><th>Intention</th>");
			for (Entity entity : alternativesOfIntention.keySet()) {
				buffer.append("<tr>").append("<td>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</td>").append("<td>");
				buffer.append(this.getIntentionType());
				buffer.append(' ');
				buffer.append(alternativesOfIntention.get(entity) ? "(negated)" : "");
				buffer.append("</td>");
				buffer.append("</tr>");
			}
			buffer.append("</table>");
		}
		if (!noHolder && holdersOfIntention != null && holdersOfIntention.size() > 0) {
			buffer.append("<br/>");
			buffer.append("<table>");
			buffer.append("<th>Holders</th><th>Negated?</th>");
			for (Entity entity : holdersOfIntention.keySet()) {
				buffer.append("<tr>").append("<td>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</td>").append("<td>");
				buffer.append(holdersOfIntention.get(entity));
				buffer.append("</td>");
				buffer.append("</tr>");
			}
			buffer.append("</table>");
		}
		if (!noRecipient && recipientsOfIntention != null && recipientsOfIntention.size() > 0) {
			buffer.append("<br/>");
			buffer.append("<table>");
			buffer.append("<th>Recipients</th><th>Negated?</th>");
			for (Entity entity : recipientsOfIntention.keySet()) {
				buffer.append("<tr>").append("<td>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</td>").append("<td>");
				buffer.append(recipientsOfIntention.get(entity));
				buffer.append("</td>");
				buffer.append("</tr>");
			}
			buffer.append("</table>");
		}
		return buffer.toString();
	}
}
