package com.aiaioo.sentiment;

import java.util.HashMap;

import com.aiaioo.sentence.Chunk;
import com.aiaioo.sentence.SimpleTextSegment;
import com.aiaioo.sentence.TextSegment;
import com.aiaioo.util.Entity;
import com.aiaioo.util.Range;

public class EntitySentiment extends Sentiment {
	protected HashMap<Entity, Sentiment> entitySentiment = new HashMap<Entity, Sentiment>();
	protected HashMap<Entity, Sentiment> holderSentiment = new HashMap<Entity, Sentiment>();

	public EntitySentiment() {
	}

	public EntitySentiment(double value) {
		super(value);
	}
	
	public EntitySentiment(double value, SentimentType type) {
		super(value, type);
	}
	
	public EntitySentiment(double value, SentimentType type, TextSegment text) {
		super(value, type, text);
	}
		
	public void addEntitySentiment(Entity entity, Sentiment sentiment) {
		entitySentiment.put(entity, sentiment);
	}

	public void addHolderSentiment(Entity entity, Sentiment sentiment) {
		holderSentiment.put(entity, sentiment);
	}

	public HashMap<Entity, Sentiment> getEntitySentiment() {
		return entitySentiment;
	}
	
	public HashMap<Entity, Sentiment> getHolderSentiment() {
		return holderSentiment;
	}
	
	public String toString() {
		StringBuffer buffer = new StringBuffer(getText().toString());
		buffer.append('\n');
		buffer.append("Sentiment: "+this.getSentimentType());
		buffer.append('\n');
		for (Entity entity : entitySentiment.keySet()) {
			buffer.append("Entity: "+entity.toString());
			buffer.append('\t');
			buffer.append(entitySentiment.get(entity).getSentimentType());
			buffer.append('\t');
			if (entitySentiment.get(entity).getHolderRelations() != null && entitySentiment.get(entity).getHolderRelations().size() > 0) {
				buffer.append("Holder:").append('[').append(' ');
				for (Chunk sentimentRelationKeys : entitySentiment.get(entity).getHolderRelations().keySet()) {
					if (!entitySentiment.get(entity).getHolderRelations().get(sentimentRelationKeys) == true) {
						buffer.append(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())));
					}
					buffer.append(' ');
				}
				buffer.append(']');
			}
			buffer.append('\t');
			if (entitySentiment.get(entity).getSentimentRelations() != null && entitySentiment.get(entity).getSentimentRelations().size() > 0) {
				buffer.append("/Sentiment:").append(' ');
				for (Chunk sentimentRelationKeys : entitySentiment.get(entity).getSentimentRelations().keySet()) {
					buffer.append(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())));
					if (entitySentiment.get(entity).getSentimentRelations().get(sentimentRelationKeys) == true) {
						buffer.append(':');
						buffer.append("negated");
					}
					buffer.append(' ');
				}
				buffer.append('/');
			}
			buffer.append('\n');
		}
		for (Entity entity : holderSentiment.keySet()) {
			buffer.append("Holder: "+entity.toString());
			buffer.append('\t');
			buffer.append(holderSentiment.get(entity).getSentimentType());
			buffer.append('\t');
			if (holderSentiment.get(entity).getSentimentRelations() != null && holderSentiment.get(entity).getSentimentRelations().size() > 0) {
				buffer.append("/Sentiment:").append(' ');
				for (Chunk sentimentRelationKeys : holderSentiment.get(entity).getSentimentRelations().keySet()) {
					buffer.append(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())));
					if (holderSentiment.get(entity).getSentimentRelations().get(sentimentRelationKeys) == true) {
						buffer.append(':');
						buffer.append("negated");
					}
					buffer.append(' ');
				}
				buffer.append('/');
			}
			buffer.append('\n');
		}
		return buffer.toString();
	}

	public String toXMLString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("<text>");
		buffer.append(escapeXMLBig(getText().toString()));
		buffer.append("</text>");
		buffer.append("<sentiment>");
		buffer.append(this.getSentimentType());
		buffer.append("</sentiment>");
		if (entitySentiment != null && entitySentiment.size() > 0) {
			buffer.append("<entities>");
			for (Entity entity : entitySentiment.keySet()) {
				buffer.append("<record>");
				buffer.append("<entity>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</entity><polarity>");
				buffer.append(entitySentiment.get(entity).getSentimentType());
				buffer.append("</polarity><intensity>");
				if (entitySentiment.get(entity).getSentimentType().equals(SentimentType.neutral))
					buffer.append("zero");
				else
					buffer.append(Math.abs(entitySentiment.get(entity).getSentimentValue()) > 3 ? "strong" : "weak");
				buffer.append("</intensity><holders>");
				if (entitySentiment.get(entity).getHolderRelations() != null && entitySentiment.get(entity).getHolderRelations().size() > 0) {
					for (Chunk sentimentRelationKeys : entitySentiment.get(entity).getHolderRelations().keySet()) {
						if (!entitySentiment.get(entity).getHolderRelations().get(sentimentRelationKeys) == true) {
							buffer.append("<holder>").append(escapeXML(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())).toString())).append("</holder>");
						}
					}
				}
				buffer.append("</holders><sentiments>");
				if (entitySentiment.get(entity).getSentimentRelations() != null && entitySentiment.get(entity).getSentimentRelations().size() > 0) {
					for (Chunk sentimentRelationKeys : entitySentiment.get(entity).getSentimentRelations().keySet()) {
						if (entitySentiment.get(entity).getSentimentRelations().get(sentimentRelationKeys) == true) {
							buffer.append("<sentiment>");
							buffer.append("<negated>");
							buffer.append(escapeXML(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())).toString()));
							buffer.append("</negated>");
							buffer.append("</sentiment>");
						} else {
							buffer.append("<sentiment>");
							buffer.append(escapeXML(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())).toString()));
							buffer.append("</sentiment>");
						}
					}
				}
				buffer.append("</sentiments>");
				buffer.append("</record>");
			}
			buffer.append("</entities>");
		}
		if (holderSentiment != null && holderSentiment.size() > 0) {
			buffer.append("<holders>");
			for (Entity entity : holderSentiment.keySet()) {
				buffer.append("<record>");
				buffer.append("<holder>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</holder><polarity>");
				buffer.append(holderSentiment.get(entity).getSentimentType());
				buffer.append("</polarity><intensity>");
				if (holderSentiment.get(entity).getSentimentType().equals(SentimentType.neutral))
					buffer.append("zero");
				else
					buffer.append(Math.abs(holderSentiment.get(entity).getSentimentValue()) > 3 ? "strong" : "weak");
				buffer.append("</intensity><sentiments>");
				if (holderSentiment.get(entity).getSentimentRelations() != null && holderSentiment.get(entity).getSentimentRelations().size() > 0) {
					for (Chunk sentimentRelationKeys : holderSentiment.get(entity).getSentimentRelations().keySet()) {
						if (holderSentiment.get(entity).getSentimentRelations().get(sentimentRelationKeys) == true) {
							buffer.append("<sentiment>");
							buffer.append("<negated>");
							buffer.append(escapeXML(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())).toString()));
							buffer.append("</negated>");
							buffer.append("</sentiment>");
						} else {
							buffer.append("<sentiment>");
							buffer.append(escapeXML(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())).toString()));
							buffer.append("</sentiment>");
						}
					}
				}
				buffer.append("</sentiments>");
				buffer.append("</record>");
			}
			buffer.append("</holders>");
		}
		return buffer.toString();
	}

	public String toWebString() {
		StringBuffer buffer = new StringBuffer(escapeXML(getText().toString()));
		buffer.append(" = [ <font color='blue'>");
		buffer.append(this.getSentimentType());
		buffer.append("</font> ]");
		buffer.append("<br/>");
		if (entitySentiment != null && entitySentiment.size() > 0) {
			buffer.append("<br/>");
			buffer.append("Entity-Level Sentiment Breakdown:");
			buffer.append("<br/>").append("<br/>");
			buffer.append("<table>");
			buffer.append("<th>Object</th><th>Polarity</th><th>Intensity</th><th>Holder</th><th>Sentiment Phrases</th>");
			for (Entity entity : entitySentiment.keySet()) {
				buffer.append("<tr>").append("<td>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</td><td>");
				buffer.append(entitySentiment.get(entity).getSentimentType());
				buffer.append("</td><td>");
				if (entitySentiment.get(entity).getSentimentType().equals(SentimentType.neutral))
					buffer.append("zero");
				else
					buffer.append(Math.abs(entitySentiment.get(entity).getSentimentValue()) > 3 ? "strong" : "weak");
				buffer.append("</td><td>");
				if (entitySentiment.get(entity).getHolderRelations() != null && entitySentiment.get(entity).getHolderRelations().size() > 0) {
					for (Chunk sentimentRelationKeys : entitySentiment.get(entity).getHolderRelations().keySet()) {
						if (!entitySentiment.get(entity).getHolderRelations().get(sentimentRelationKeys) == true) {
							buffer.append(escapeXML(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())).toString()));
						}
						buffer.append(' ');
					}
				}
				buffer.append("</td><td>");
				if (entitySentiment.get(entity).getSentimentRelations() != null && entitySentiment.get(entity).getSentimentRelations().size() > 0) {
					for (Chunk sentimentRelationKeys : entitySentiment.get(entity).getSentimentRelations().keySet()) {
						buffer.append(escapeXML(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())).toString()));
						if (entitySentiment.get(entity).getSentimentRelations().get(sentimentRelationKeys) == true) {
							buffer.append(':');
							buffer.append("negated");
						}
						buffer.append(' ');
					}
				}
				buffer.append("</td>");
				buffer.append("</tr>");
			}
			buffer.append("</table>");
		}
		if (holderSentiment != null && holderSentiment.size() > 0) {
			buffer.append("<br/>");
			buffer.append("Opinion-Holder-Level Sentiment Breakdown:");
			buffer.append("<br/>").append("<br/>");
			buffer.append("<table>");
			buffer.append("<th>Holder</th><th>Polarity</th><th>Intensity</th><th>Sentiment Phrases</th>");
			for (Entity entity : holderSentiment.keySet()) {
				buffer.append("<tr>").append("<td>");
				buffer.append(escapeXML(entity.toString()));
				buffer.append("</td><td>");
				buffer.append(holderSentiment.get(entity).getSentimentType());
				buffer.append("</td><td>");
				if (holderSentiment.get(entity).getSentimentType().equals(SentimentType.neutral))
					buffer.append("zero");
				else
					buffer.append(Math.abs(holderSentiment.get(entity).getSentimentValue()) > 3 ? "strong" : "weak");
				buffer.append("</td><td>");
				if (holderSentiment.get(entity).getSentimentRelations() != null && holderSentiment.get(entity).getSentimentRelations().size() > 0) {
					for (Chunk sentimentRelationKeys : holderSentiment.get(entity).getSentimentRelations().keySet()) {
						buffer.append(escapeXML(segment(sentimentRelationKeys.getTextSegment(),(sentimentRelationKeys.getRange())).toString()));
						if (holderSentiment.get(entity).getSentimentRelations().get(sentimentRelationKeys) == true) {
							buffer.append(':');
							buffer.append("negated");
						}
						buffer.append(' ');
					}
				}
				buffer.append("</td>");
				buffer.append("</tr>");
			}
			buffer.append("</table>");
		}
		return buffer.toString();
	}

	private TextSegment segment(TextSegment textSegment, Range range) {
		if (range.getStart() < 0 || range.getEnd() > textSegment.size())
			return textSegment;
		else
			return new SimpleTextSegment(textSegment.getWords().subList((int)range.getStart(), (int)range.getEnd()+1));
	}
}
