package com.aiaioo.sentiment;

import java.util.HashMap;

import com.aiaioo.sentence.Chunk;
import com.aiaioo.sentence.TextSegment;

public class Sentiment {
	private double sentimentValue = 0;
	private SentimentType type;
	private TextSegment text;
	private HashMap<Chunk, Boolean> associatedSentiment, associatedHolder;

	public Sentiment() {
	}

	public Sentiment(double value) {
		this.sentimentValue = value;
	}
	
	public Sentiment(SentimentType type) {
		this.type = type;
		if (type == SentimentType.positive)
			this.sentimentValue = 2;
		else if (type == SentimentType.negative)
			this.sentimentValue = -2;
		else
			this.sentimentValue = 0;
	}
	
	public Sentiment(double value, SentimentType type) {
		this.sentimentValue = value;
		this.type = type;
	}
	
	public Sentiment(double value, SentimentType type, TextSegment text) {
		this.sentimentValue = value;
		this.type = type;
		this.text = text;
	}
	
	public SentimentType getSentimentType() {
		if (this.type == null)
			return (this.sentimentValue > 0.5) ? SentimentType.positive : SentimentType.negative;
		else
			return this.type;
	}

	public void setSentimentType(SentimentType type) {
		this.type = type;
	}
	
	public double getSentimentValue() {
		return sentimentValue;
	}

	public void setSentimentValue(double sentimentValue) {
		this.sentimentValue = sentimentValue;
	}

	public TextSegment getText() {
		return text;
	}

	public void setText(TextSegment text) {
		this.text = text;
	}

	public void setSentimentRelations(HashMap<Chunk, Boolean> associatedSentiment) {
		this.associatedSentiment = associatedSentiment;
	}

	public void setHolderRelations(HashMap<Chunk, Boolean> associatedHolder) {
		this.associatedHolder = associatedHolder;
	}

	public HashMap<Chunk, Boolean> getSentimentRelations() {
		return this.associatedSentiment;
	}
	
	public HashMap<Chunk, Boolean> getHolderRelations() {
		return this.associatedHolder;
	}
	
	public double getSentimentValueNormalized() {
		return sentimentValue;
	}
	
	public String toString() {
		return getText().toString();
	}
	
	public String toXMLString() {
		return escapeXML(toString());
	}
	
	public String toWebString() {
		return escapeXML(toString());
	}
	
	public String escapeXML(String myString) {
		myString = myString.replaceAll("&", "&amp;");
		myString = myString.replaceAll("<", "&lt;");
		myString = myString.replaceAll(">", "&gt;");
		myString = myString.replaceAll("'", "&apos;");
		myString = myString.replaceAll("\"", "&quot;");
		return myString;
	}

	public String escapeXMLBig(String myString) {
		return "<![CDATA[" + myString.replaceAll("]]>", "]]>]]><![CDATA[") + "]]>";
	}

}
